import os

os.environ["SDG_NDARRAY_CACHE_ROOT"] = "/tmp/sdgx/ndarray_cache"
import shutil

import pytest

from sdgx.data_connectors.csv_connector import CsvConnector
from sdgx.data_loader import DataLoader
from sdgx.data_models.metadata import Metadata
from sdgx.utils import download_demo_data

_HERE = os.path.dirname(__file__)

# Cache it locally for rerun the tests
DATA_DIR = os.path.join(_HERE, "dataset")


@pytest.fixture
def demo_single_table_path():
    yield download_demo_data(DATA_DIR)


@pytest.fixture
def cacher_kwargs(tmp_path):
    cache_dir = tmp_path / "cache"
    yield {"cache_dir": cache_dir.as_posix()}
    shutil.rmtree(cache_dir, ignore_errors=True)


@pytest.fixture
def demo_single_table_data_connector(demo_single_table_path):
    yield CsvConnector(
        path=demo_single_table_path,
    )


@pytest.fixture
def demo_single_table_data_loader(demo_single_table_data_connector, cacher_kwargs):
    d = DataLoader(demo_single_table_data_connector, cacher_kwargs=cacher_kwargs)
    yield d
    d.finalize()


@pytest.fixture
def demo_single_table_metadata(demo_single_table_data_loader):
    yield Metadata.from_dataloader(demo_single_table_data_loader)



"""
Example for fit and save a model, then load it for sampling.
"""

import time
from pathlib import Path

from sdgx.data_connectors.csv_connector import CsvConnector
from sdgx.models.ml.single_table.ctgan import CTGANSynthesizerModel
from sdgx.synthesizer import Synthesizer
from sdgx.utils import download_demo_data

_HERE = Path(__file__).parent

# This will download demo data to ./dataset
dataset_csv = download_demo_data()

# Create data connector for csv file
data_connector = CsvConnector(path=dataset_csv)

# Initialize synthesizer, use CTGAN model
synthesizer = Synthesizer(
    model=CTGANSynthesizerModel(epochs=1),  # For quick demo
    data_connector=data_connector,
)

# Fit and save model
synthesizer.fit()
date = time.strftime("%Y%m%d-%H%M%S")
save_dir = _HERE / f"./ctgan-{date}-model"
synthesizer.save(save_dir)

# Load model, then sample
synthesizer = Synthesizer.load(save_dir, model=CTGANSynthesizerModel)
sampled_data = synthesizer.sample(1000)
print(sampled_data)



from sdgx.data_processors.base import DataProcessor


class Transformer(DataProcessor):
    """
    Base class for transformers.

    Transformer is used to transform table data from one format to another.
    For example, encode discrete column into one hot encoding.

    To achieve that, Transformer can use :ref:`Formatter` and :ref:`Inspector` to help.
    """



from __future__ import annotations

from pathlib import Path

import pandas as pd

from sdgx.data_loader import DataLoader
from sdgx.data_models.metadata import Metadata


class SynthesizerModel:
    def fit(self, metadata: Metadata, dataloader: DataLoader, *args, **kwargs):
        """
        Fit the model using the given metadata and dataloader.

        Args:
            metadata (Metadata): The metadata to use.
            dataloader (DataLoader): The dataloader to use.
        """
        raise NotImplementedError

    def sample(self, count: int, *args, **kwargs) -> pd.DataFrame:
        """
        Sample data from the model.

        Args:
            count (int): The number of samples to generate.

        Returns:
            pd.DataFrame: The generated data.
        """

        raise NotImplementedError

    def save(self, save_dir: str | Path):
        """
        Dump model to file.

        Args:
            save_dir (str | Path): The directory to save the model.
        """

        raise NotImplementedError

    @classmethod
    def load(cls, save_dir: str | Path) -> "SynthesizerModel":
        """
        Load model from file.

        Args:
            save_dir (str | Path): The directory to load the model from.
        """
        raise NotImplementedError



from __future__ import annotations

from typing import Any

from sdgx import data_processors
from sdgx.data_processors import extension
from sdgx.data_processors.base import DataProcessor
from sdgx.data_processors.extension import project_name as PROJECT_NAME
from sdgx.manager import Manager


class DataProcessorManager(Manager):
    register_type = DataProcessor
    project_name = PROJECT_NAME
    hookspecs_model = extension

    @property
    def registed_data_processors(self):
        return self.registed_cls

    def load_all_local_model(self):
        self._load_dir(data_processors.formatters)
        self._load_dir(data_processors.generators)
        self._load_dir(data_processors.samplers)
        self._load_dir(data_processors.transformers)

    def init_data_processor(self, processor_name, **kwargs: dict[str, Any]) -> DataProcessor:
        return self.init(processor_name, **kwargs)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

sys.path.insert(0, os.path.abspath(".."))

# -- Project information -----------------------------------------------------

project = "Synthetic Data Generator"
copyright = "2023, hitsz-ids"
author = "hitsz-ids"

# The full version, including alpha/beta/rc tags
from sdgx import __version__

release = __version__

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx.ext.viewcode",
    "sphinx.ext.todo",
    "sphinx.ext.autosectionlabel",
    "sphinx_click",
    "sphinxcontrib.autodoc_pydantic",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "pydata_sphinx_theme"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

todo_include_todos = True
html_logo = "_static/sdg_logo.png"
html_theme_options = {
    "github_url": "https://github.com/hitsz-ids/synthetic-data-generator",
    "use_edit_page_button": True,
}

html_context = {
    "github_user": "hitsz-ids",
    "github_repo": "synthetic-data-generator",
    "github_version": "main",
    "doc_path": "docs/source",
}



from __future__ import annotations

import pandas as pd

from sdgx.data_models.metadata import Metadata


class DataProcessor:
    """
    Base class for data processors.
    """

    def fit(self, metadata: Metadata | None = None):
        """Fit the data processor.

        Called before ``convert`` and ``reverse_convert``.

        Args:
            metadata (Metadata, optional): Metadata. Defaults to None.
        """
        return

    def convert(self, raw_data: pd.DataFrame) -> pd.DataFrame:
        """Convert raw data into processed data.

        Args:
            raw_data (pd.DataFrame): Raw data

        Returns:
            pd.DataFrame: Processed data
        """
        return raw_data

    def reverse_convert(self, processed_data: pd.DataFrame) -> pd.DataFrame:
        """Convert processed data into raw data.

        Args:
            processed_data (pd.DataFrame): Processed data

        Returns:
            pd.DataFrame: Raw data
        """
        return processed_data



from __future__ import annotations

from typing import Any

import pandas as pd


class Inspector:
    """
    Base Inspector class

    Inspector is used to inspect data and generate metadata automatically.

    Parameters:
        ready (bool): Ready to inspect, maybe all fields are fitted, or indicate if there is more data, inspector will be more precise.
    """

    def __init__(self, *args, **kwargs):
        self.ready: bool = False

    def fit(self, raw_data: pd.DataFrame):
        """Fit the inspector.

        Args:
            raw_data (pd.DataFrame): Raw data
        """
        return

    def inspect(self) -> dict[str, Any]:
        """Inspect raw data and generate metadata."""



from __future__ import annotations

from typing import Generator

import pandas as pd

from sdgx.cachers.base import Cacher
from sdgx.cachers.extension import hookimpl
from sdgx.data_connectors.base import DataConnector
from sdgx.exceptions import CacheError
from sdgx.log import logger


class MemoryCache(Cacher):
    """
    Cacher that cache data in memory

    Only useful for testing and small dataset for performance.

    Args:
        blocksize (int): The blocksize of the cache.

    """

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.cache = {}

    def is_cached(self, offset: int) -> bool:
        return offset in self.cache

    def clear_invalid_cache(self):
        self.cache.clear()

    def _refresh(self, offset: int, data: pd.DataFrame) -> None:
        if len(data) < self.blocksize:
            self.cache[offset] = data
        elif len(data) > self.blocksize:
            for i in range(0, len(data), self.blocksize):
                self.cache[offset + i] = data[i : i + self.blocksize]
        else:
            self.cache[offset] = data

    def load(self, offset: int, chunksize: int, data_connector: DataConnector) -> pd.DataFrame:
        if chunksize % self.blocksize != 0:
            raise CacheError(
                "chunksize must be multiple of blocksize, current chunksize is {} and blocksize is {}".format(
                    chunksize, self.blocksize
                )
            )

        if chunksize != self.blocksize:
            logger.warning("chunksize must be equal to blocksize, may cause performance issue.")

        if self.is_cached(offset):
            cached_data = self.cache[offset]
            if len(cached_data) >= chunksize:
                return cached_data[:chunksize]
            return cached_data

        data = data_connector.read(offset=offset, limit=max(self.blocksize, chunksize))
        if data is None:
            return data
        self._refresh(offset, data)
        if len(data) < chunksize:
            return data
        return data[:chunksize]

    def iter(
        self, chunksize: int, data_connector: DataConnector
    ) -> Generator[pd.DataFrame, None, None]:
        offset = 0
        while True:
            data = self.load(offset, chunksize, data_connector)
            if data is not None and len(data) == 0:
                break
            yield data
            offset += len(data)


@hookimpl
def register(manager):
    manager.register("MemoryCache", MemoryCache)



import os
from collections import namedtuple
from io import StringIO
from typing import List, Optional

import numpy as np
import pandas as pd
from joblib import Parallel, delayed

from sdgx.data_processors.transformers.base import Transformer
from sdgx.models.components.sdv_rdt.transformers import (
    ClusterBasedNormalizer,
    OneHotEncoder,
)

SpanInfo = namedtuple("SpanInfo", ["dim", "activation_fn"])
ColumnTransformInfo = namedtuple(
    "ColumnTransformInfo",
    ["column_name", "column_type", "transform", "output_info", "output_dimensions"],
)


def get_csv_column(input_path, column_name):
    """按 column 形式读取 csv，返回 pd.DataFrame 的单列数据

    输入参数:
        input_path (str):
            作为输入的 csv 路径
        column_name (str)：
            需要提取的 csv 列名

    返回对象说明:
        namedtuple对象 (pd.DataFrame):
            返回单个 ``pd.DataFrame`` 对象
    """
    df_col = pd.read_csv(input_path, usecols=[column_name])
    return df_col


def get_csv_column_count(input_path, id_col=False):
    """get the column number from csv, the input is the path

    Args:
        input_path (str):
            the path of csv file
        id_col (bool):
            whether csv file contains id column,
            if true, the count of column will minus by 1

    Returns:
        cnt (int):
            the count of column in csv
    """
    first_line = None
    with open(input_path) as f:
        first_line = f.readline()
    cnt = len(first_line.split(","))
    if id_col:
        cnt = cnt - 1
    return cnt


# OPTIMIZE SDG 重写的 Data Transformer
class DataTransformer(Transformer):
    """OPTIMIZE SDG 重写的 Data Transformer

    应对大数据（数据 > 内存）情况下的 Transformer 解决方案（试行）
        - 对于连续列：使用 BayesianGMM 对连续列建模并标准化为标量 [0, 1] 和向量。
        - 对于离散列：使用 scikit-learn OneHotEncoder 进行编码。

    本例的实现主要是为了支持大数据情况，并支持将 transform 的数据结果 以及 transformer
    所需的各类元数据 dump 到硬盘中，以实现较少的内存消耗。
    """

    def __init__(self, max_clusters=10, weight_threshold=0.005):
        """Create a data transformer.

        输入参数:
            max_clusters (int):
                Maximum number of Gaussian distributions in Bayesian GMM.
            weight_threshold (float):
                Weight threshold for a Gaussian distribution to be kept.
        """
        self._max_clusters = max_clusters
        self._weight_threshold = weight_threshold

    def _fit_continuous(self, column_name):
        """针对连续特征，训练一个贝叶斯 GMM， 用于 transform。
            该方法暂时不需要做进一步修改

        输入参数:
            data (pd.DataFrame):
                一个 pd.DataFrame 对象，但是只含有单列数据

        返回对象说明:
            namedtuple对象:
                返回单个 ``ColumnTransformInfo`` 对象
        """
        data = get_csv_column(self.raw_data_path, column_name)

        # column_name = data.columns[0]

        gm = ClusterBasedNormalizer(
            missing_value_generation="from_column", max_clusters=min(len(data), 10)
        )
        gm.fit(data, column_name)
        num_components = sum(gm.valid_component_indicator)

        # 使用 transform 对象存储单个列的 transformer
        return ColumnTransformInfo(
            column_name=column_name,
            column_type="continuous",
            transform=gm,
            output_info=[SpanInfo(1, "tanh"), SpanInfo(num_components, "softmax")],
            output_dimensions=1 + num_components,
        )

    def _fit_discrete(self, column_name):
        """针对离散特征：使用一个 one hot encoder 进行编码.
            该方法暂时不需要做进一步修改

        输入参数:
            data (pd.DataFrame):
                一个 pd.DataFrame 对象，但是只含有单列数据
            column_name (str)：
                需要进行 fit 操作的列名

        返回对象说明:
            namedtuple 对象:
                返回单个 ``ColumnTransformInfo`` 对象
        """
        data = get_csv_column(self.raw_data_path, column_name)
        # column_name = data.columns[0]
        ohe = OneHotEncoder()
        ohe.fit(data, column_name)
        num_categories = len(ohe.dummies)

        # 使用 transform 对象存储单个列的 transformer
        return ColumnTransformInfo(
            column_name=column_name,
            column_type="discrete",
            transform=ohe,
            output_info=[SpanInfo(num_categories, "softmax")],
            output_dimensions=num_categories,
        )

    # `fit` 是 transform 的入口方法
    def fit(self, raw_data_path, discrete_columns: Optional[List] = None, csv_cache_N=1000):
        """进行 DataTransformer 的 Fit 操作

        针对连续变量，使用 ``ClusterBasedNormalizer`` 进行 transform；
        对于离散变量，使用 ``OneHotEncoder``  进行 transform。

        此步骤还对矩阵数据和跨度信息中的#columns 进行计数操作。

        输入参数:
            raw_data_path (str):
                目前仅接受 csv 文件作为输入。
            discrete_columns (list):
                离散列名称，需要与 csv 文件的列名对应名，
                默认该参数为 [] 。
            csv_cache_N (int):
                为了减少内存，使用缓存 csv 的行数。
        """
        # raw data 需要从 path 中获取
        # raw_data = None
        self.raw_data_path = raw_data_path
        self.output_info_list = []
        self.output_dimensions = 0
        self.csv_cache_N = csv_cache_N
        # self.dataframe = False
        if not discrete_columns:
            discrete_columns = []

        # 对 data frame 的检测，如果不是 data frame ，还是转换为了 dataframe 进行操作
        # 但是经过搜索 此处仅对 后续一个逻辑又变化，所以我们直接把 self. dataframe 设置为 False
        # 在未来处理的时候，单独载入 data frame
        # if not isinstance(raw_data, pd.DataFrame):
        #     self. dataframe = False
        #     # work around for RDT issue #328 Fitting with numerical column names fails
        #     discrete_columns = [str(column) for column in discrete_columns]
        #     column_names = [str(num) for num in range(raw_data.shape[1])]
        #     raw_data = pd.DataFrame(raw_data, columns=column_names)

        # 对 discrete_columns 的处理还是保留了下来
        discrete_columns = [str(column) for column in discrete_columns]

        # 对 raw_data_path 增加检查
        #   - 是否存在，使用 os.path.exists 判断
        #   - 是否是 csv ，目前仅做名称上的检查
        if not os.path.exists(raw_data_path):
            raise FileNotFoundError("Input csv NOT Found.")
        if not "csv" in raw_data_path:
            raise ValueError("Input Data is NOT CSV.")

        # raw data columns 在原有的实现方法中使用了 raw_data.columns
        # 这么做的问题就在于 df 直接载入了所有数据
        # 为了减少内存消耗，我们通过读取 csv 的前 x 行，并借助原有代码完成相应功能的计算
        cache_csv_str = ""
        with open(raw_data_path, "r") as f:
            for _ in range(self.csv_cache_N + 1):
                cache_csv_str += f.readline()
        # 从 str 创建 data frame
        cache_io = StringIO(cache_csv_str)
        raw_data_cache = pd.read_csv(cache_io)

        # get columns
        self.column_names = [str(num) for num in range(raw_data_cache.shape[1])]

        self._column_raw_dtypes = raw_data_cache.infer_objects().dtypes
        self._column_transform_info_list = []

        for column_name in raw_data_cache.columns:
            if column_name in discrete_columns:
                column_transform_info = self._fit_discrete(column_name)
            else:
                column_transform_info = self._fit_continuous(column_name)

            self.output_info_list.append(column_transform_info.output_info)
            self.output_dimensions += column_transform_info.output_dimensions
            self._column_transform_info_list.append(column_transform_info)

    def _transform_continuous(self, column_transform_info, data):
        column_name = data.columns[0]
        flattened_column = data[column_name].to_numpy().flatten()
        data = data.assign(**{column_name: flattened_column})
        transformed_data = column_transform_info.transform.transform(data)

        output_dimensions = column_transform_info.output_dimensions
        output = np.zeros((len(transformed_data), output_dimensions))
        output[:, 0] = transformed_data[f"{column_name}.normalized"].to_numpy()
        index = transformed_data[f"{column_name}.component"].to_numpy().astype(int)
        output[np.arange(index.size), index + 1] = 1.0

        return output

    def _transform_discrete(self, column_transform_info, data):
        ohe = column_transform_info.transform
        return ohe.transform(data).to_numpy()

    def _synchronous_transform(self, input_data_path, column_transform_info_list, output_path):
        """Take a csv path, and transform columns synchronous.

        Outputs a list with Numpy arrays to disk.
        """

        loop = True
        # has_write_header = True
        # use iterator = True
        reader = pd.read_csv(input_data_path, iterator=True, chunksize=1000000)
        while loop:
            column_data_list = []
            # get raw data
            raw_data = None
            try:
                raw_data = reader.get_chunk()
            except StopIteration:
                loop = False

            # break if loop is iteration is end
            if not loop:
                break

            """
            # write csv header to file
            if has_write_header:
                f = open(output_path, "w")
                f.write(",".join(raw_data.columns.to_list()) + '\n')
                f.close()
                has_write_header = False
            """

            # transform data here
            for column_transform_info in column_transform_info_list:
                column_name = column_transform_info.column_name
                # 获取列数据
                data = raw_data[[column_name]]
                if column_transform_info.column_type == "continuous":
                    column_data_list.append(self._transform_continuous(column_transform_info, data))
                else:
                    column_data_list.append(self._transform_discrete(column_transform_info, data))

            # 追加写到 output path 吧
            chunk_array = np.concatenate(column_data_list, axis=1).astype(float)
            # file object
            f = open(output_path, "a")
            np.savetxt(f, chunk_array, fmt="%g", delimiter=",")
            f.close()
        # end while
        # return column_data_list

    def _parallel_transform(self, raw_data, column_transform_info_list):
        """Take a Pandas DataFrame and transform columns in parallel.

        Outputs a list with Numpy arrays.
        """
        processes = []
        for column_transform_info in column_transform_info_list:
            column_name = column_transform_info.column_name
            data = raw_data[[column_name]]
            process = None
            if column_transform_info.column_type == "continuous":
                process = delayed(self._transform_continuous)(column_transform_info, data)
            else:
                process = delayed(self._transform_discrete)(column_transform_info, data)
            processes.append(process)

        return Parallel(n_jobs=-1)(processes)

    # TODO function to finish
    def transform(self, input_data_path, output_data_path):
        """Take raw data and output a matrix data."""
        # 从 disk 中读取
        raw_data = None

        #  check the parameter `input_data_path`
        if not os.path.exists(input_data_path):
            raise FileNotFoundError("Input csv NOT Found.")
        if not "csv" in input_data_path:
            raise ValueError("Input Data is NOT CSV.")

        # check the parameter `output data path`
        if not "csv" in output_data_path:
            raise ValueError("Input Data is NOT CSV.")

        # get the csv shape column count
        # import func from utils
        column_cnt = get_csv_column_count(input_data_path)

        # Only use parallelization with larger data sizes.
        # Otherwise, the transformation will be slower.
        # if column_cnt < 500:
        # column_data_list = self._synchronous_transform(
        self._synchronous_transform(
            input_data_path, self._column_transform_info_list, output_data_path
        )
        # else:
        #     column_data_list = self._parallel_transform(
        #         input_data_path,
        #         self._column_transform_info_list)

        # return np.concatenate(column_data_list, axis=1).astype(float)
        return output_data_path

    def _inverse_transform_continuous(self, column_transform_info, column_data, sigmas, st):
        gm = column_transform_info.transform
        data = pd.DataFrame(column_data[:, :2], columns=list(gm.get_output_sdtypes()))
        data[data.columns[1]] = np.argmax(column_data[:, 1:], axis=1)
        if sigmas is not None:
            selected_normalized_value = np.random.normal(data.iloc[:, 0], sigmas[st])
            data.iloc[:, 0] = selected_normalized_value

        return gm.reverse_transform(data)

    def _inverse_transform_discrete(self, column_transform_info, column_data):
        ohe = column_transform_info.transform
        data = pd.DataFrame(column_data, columns=list(ohe.get_output_sdtypes()))
        return ohe.reverse_transform(data)[column_transform_info.column_name]

    def inverse_transform(self, data, sigmas=None, output_path=None, write_header=True):
        """Take matrix data and output raw data.

        Output uses the same type as input to the transform function.
        Either np array or pd dataframe.
        """
        st = 0
        recovered_column_data_list = []
        column_names = []
        for column_transform_info in self._column_transform_info_list:
            dim = column_transform_info.output_dimensions
            column_data = data[:, st : st + dim]
            if column_transform_info.column_type == "continuous":
                recovered_column_data = self._inverse_transform_continuous(
                    column_transform_info, column_data, sigmas, st
                )
            else:
                recovered_column_data = self._inverse_transform_discrete(
                    column_transform_info, column_data
                )

            recovered_column_data_list.append(recovered_column_data)
            column_names.append(column_transform_info.column_name)
            st += dim

        recovered_data = np.column_stack(recovered_column_data_list)
        recovered_data = pd.DataFrame(recovered_data, columns=column_names).astype(
            self._column_raw_dtypes
        )

        # input is csv, self.dataframe will always be true
        # if not self.dataframe:
        #     recovered_data = recovered_data.to_numpy()
        if output_path is not None:
            f = open(output_path, "w")
            # write csv header to file
            if write_header:
                # f = open(output_path, "w")
                # recovered_data's type is pd.DataFrame
                f.write(",".join(recovered_data.columns.to_list()) + "\n")
            f.close()
            recovered_data.to_csv(output_path, index=False)

        return recovered_data

    def convert_column_name_value_to_id(self, column_name, value):
        """Get the ids of the given `column_name`."""
        discrete_counter = 0
        column_id = 0
        for column_transform_info in self._column_transform_info_list:
            if column_transform_info.column_name == column_name:
                break
            if column_transform_info.column_type == "discrete":
                discrete_counter += 1

            column_id += 1

        else:
            raise ValueError(f"The column_name `{column_name}` doesn't exist in the data.")

        ohe = column_transform_info.transform
        data = pd.DataFrame([value], columns=[column_transform_info.column_name])
        one_hot = ohe.transform(data).to_numpy()[0]
        if sum(one_hot) == 0:
            raise ValueError(f"The value `{value}` doesn't exist in the column `{column_name}`.")

        return {
            "discrete_column_id": discrete_counter,
            "column_id": column_id,
            "value_id": np.argmax(one_hot),
        }



# # 本例用于测试 transformer 对象的正常使用

# import os
# import shutil
# import sys

# import pytest

# _HERE = os.path.dirname(__file__)
# sys.path.append(os.getcwd())

# from sdgx.data_processors.transformers.transform import DataTransformer

# # from sdgx.data_process.transform.transformer_opt import DataTransformer
# from sdgx.utils.io.csv_utils import *


# def test_transformer_original():
#     demo_data, discrete_cols = get_demo_single_table()
#     ctgan_transformer = DataTransformer()
#     ctgan_transformer.fit(demo_data, discrete_cols)
#     transformed_data = ctgan_transformer.transform(demo_data)


# """
# def test_transformer_opt():
#     # 测试经过内存优化之后的 transformer
#     demo_data_path = "./dataset/adult.csv"
#     _, discrete_cols = get_demo_single_table()
#     opt_transformer = DataTransformer()
#     opt_transformer.fit(demo_data_path, discrete_cols)
#     opt_transformer.transform(demo_data_path, "./output_tmp.csv")
#     # inverse transforme
#     x = pd.read_csv("./output_tmp.csv")
#     x = x.to_numpy()
#     inverse_x = opt_transformer.inverse_transform(x)
#     inverse_x = opt_transformer.inverse_transform(
#         x, output_path="inverse_tmp.csv", write_header=True
#     )

#     # remove tmp files
#     shutil.os.remove("inverse_tmp.csv")
#     shutil.os.remove("output_tmp.csv")
#     pass
# """


# if __name__ == "__main__":
#     pytest.main(["-vv", "-s", __file__])
